%% Simplext example of FP cavity

%{
% examples
fp = FPbasic;                   % define FP
fp.lock;                        % lock FP
fp.power('ITM-HR-o');           % power of outgioing field from ITM HR
fp.roundtripLoss('ITM-HR-o')    % round trip loss
fp.gaussFit('ITM-HR-o')         % gaussfit of a field
fp.HGCoef('ITM-HR-o',0:1,0:1)   % HG(0,0), (1,0), (0,1), (1,1) amplitude
fp.printAll                     % print all information about fp
% field of incoming field to ETM HR 
% E(j,i) is the amplitude of the field at (x,y) = (x(i), y(j))
[E,x] = fp.getField('ETM-HR-i');
semilogy( x.^2, abs(E(end/2,:)).^2 )
%}

classdef FPbasic < FFTIFOAcc
    
    methods
        function obj = FPbasic( varargin )
            obj@FFTIFOAcc( varargin{:} );
        end

        function defineIFO( obj, varargin )
            %% add optics
            % laser source
            obj.addLaser( 'laser' );            
            % ITM
            obj.addMirror( 'ITM', 'invRoC', 1/1934, 'Aperture', 0.34, 'T', 0.014, 'Thick', 0.2 );
            % ETM
            obj.addMirror( 'ETM', 'invRoC', 1/2245, 'Aperture', 0.34, 'T', 5e-6 );
            
            %% define connections among optics
            obj.addProp( '', 'laser', 'ITM-AR' );
            obj.addProp( 'FPprop', 'ITM-HR', 'ETM-HR', 4000);
        end
    end
end
