%
% LLO DRFPM IFO
%
%{
sprintf('cold state LLO without point absorber')
llo = LLODRFPMAcc('res', 2e-3, 'armPower', 0 ); llo.lock('errorLimit',1e-3);
llo.gaussFit('ETMY-HR-o'), lossXarm=llo.roundtripLoss('ETMX-HR-o'), lossYarm=llo.roundtripLoss('ETMY-HR-o'), PRG=llo.power('BS-HRx-o')

sprintf('LLO with a point absorber on ETMY at (2cm,0cm) 50mW absorption')
llo = LLODRFPMAcc('res', 2e-3, 'armPower', 0, 'xPoY', 0.02, 'yPoY', 0, 'PpointEY', 0.05 ); llo.lock('errorLimit',1e-3);
llo.gaussFit('ETMY-HR-o'), lossXarm=llo.roundtripLoss('ETMX-HR-o'), lossYarm=llo.roundtripLoss('ETMY-HR-o'), PRG=llo.power('BS-HRx-o')

sprintf('LLO with a point absorber on ETMY at (2cm,0cm) 50mW absorption, beam tilted on ETMY to (-2cm,0), away from the point absorber')
llo = LLODRFPMAcc('res', 2e-3, 'armPower', 0, 'xPoY', 0.02, 'yPoY', 0, 'PpointEY', 0.05, 'posEYX', -0.02); llo.lock('errorLimit',1e-3);
llo.gaussFit('ETMY-HR-o'), lossXarm=llo.roundtripLoss('ETMX-HR-o'), lossYarm=llo.roundtripLoss('ETMY-HR-o'), PRG=llo.power('BS-HRx-o')

sprintf('LLO with two point absorbers, at (2cm,0) 30mW, and (-1mc, -1cm) 50mW, beam tilted on ETMY to (2cm,0)')
llo = LLODRFPMAcc('res', 2e-3, 'armPower', 0, 'xPoY', [0.02,-0.01], 'yPoY', [0,-0.01], 'PpointEY', [0.03, 0.05], 'posEYX', 0.02); llo.lock('errorLimit',1e-3);
llo.gaussFit('ETMY-HR-o'), lossXarm=llo.roundtripLoss('ETMX-HR-o'), lossYarm=llo.roundtripLoss('ETMY-HR-o'), PRG=llo.power('BS-HRx-o')
%}

%
%{
            dPRC_E1300128 = 57.656;
            SB_9MHz = 1.75 * ConstantTK.c / dPRC_E1300128;
%}

classdef LLODRFPMAcc < FFTIFOAcc
    methods
        function obj = LLODRFPMAcc( varargin )
            obj@FFTIFOAcc( varargin{:} );
        end
        
        function defineIFO( obj, varargin )
                             
            dirName = './Data0/';            
            DCoffset = 10e-12;
            % DCoffset = 29e-12;
            
            tind = 90;
            uniptX = 0;
            uniptY = 0;
            DInvREY = 0;
            
            mapOP = 1;
            mapHR = 1;
            ITMAbs = ' ';
            fitR = -0.16;  % firR < 0 : subtract power, not tilt. firR = 0 : only tilt
            
            ITMXID = 4; ITMYID = 8; 
            % old ETMs
            % ETMXID = 7; ETMYID = 9;
            % new ETMs
            ETMXID = 10; ETMYID = 15;
            
            CPXID = 6; CPYID = 3;
            
            BSID = 2;

            useBSbaf = true;
            
            extraArmLoss = 28.5e-6;
            rmloss = 1;
            ITMloss = 1;
            angleScale = 1;
            armScale = 1;
            BSScale = 1;
            
            TMMap = 1;  % if 0, no test mass maps used, and extra loss is added manually
            % focal length in ITM to correct to define mode bases
            invFocalY =  1/50e3 * 0;
            invFocalX =  1/50e3 * 0; 
            
            % PRC length correction
            dPRC23Corr = 0.00291777 + 0.00644412;
            dSRC23Corr = 0.00890212;
            
            Atm = 0.326;
            Apr3 = 0.262;
            Abs = 0.3671;
            dBS = 0.06;
            Apr1 = -8;
            Apr2 = 0.07;
            ASRM = 0.02;
            ASRM2 = 0.08;
            ASRM3 = 0.260;
            Asym = 1;
            res = 2e-3;
            T_ITM = 1.48e-2;  % This is LLO value
            remC = 0;
            Larm = 3994.5;
            
            PRCLoss = 150e-6*0;
            
            % beam positions on ITM and ETM
            % ** = X or Y arm, x coordinate or y coordinate
            % posIXY : ITM of X arm, y coordinate
            posIXX = 0; posIXY = 0;  posIYX = 0; posIYY = 0;
            posEXX = 0; posEXY = 0;  posEYX = 0; posEYY = 0;

            ETMRoC0 = 2245;
            
            TSRM = 0.3603;
            phiSRC = pi/2;
            
            remCurv = false;
            remTilt = false;
            
            absIX = 0.7e-6;
            absIY = 0.7e-6;
            absEX = 0.7e-6;
            absEY = 0.7e-6;
            absCP = 1;
            remAst = 0;
            
            armPower = 0;  % was 250e3 when v1 of G1900056-v1 was calculated
            
            % For validation
            %{
            T_ITM = 0.014;
            TSRM = 0.2;
            TMMap = 0;
            BSID = 0; 
            angleScale = 0; 
            extraArmLoss = 30e-6;
             %}
           
            DinvRoCETM = 0;
            
            wI = 0.053;
            wE = 0.062;
            
            BFdx = 0; BFdy = 0;
            BSBFX = 0.105;
            BSBFY = 0.13;

            xPoX = 0; yPoX = 0; PpointEX = 0; absSizeX = 0.001;
            xPoY = 0; yPoY = 0; PpointEY = 0; absSizeY = 0.001;
            
            mw = 0;
            mr = 0.1;

            mNI = 1;
            mNE = 1;
            
            pwr1DY = 0;
            oneDdat = 'FS_heat_10W_120.txt';
            FO0 = 0;
            NFO = 0;
            invRoCBS = 0;
            
            thetaXI = 0; phiXI = 0; thetaXE = 0; phiXE = 0;
            thetaYI = 0; phiYI = 0; thetaYE = 0; phiYE = 0;

            % Interacive parameter setting
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            PcoatIX = armPower * absIX;
            PcoatIY = armPower * absIY;
            PcoatEX = armPower * absEX;
            PcoatEY = armPower * absEY;
            
            Atm = Atm * armScale;
            Apr3 = Apr3 * armScale;
            Abs = Abs * BSScale;

            dPRM = 0.0749;
            dPRC12 = 16.6107; dPRC23 = 16.1647 + dPRC23Corr; dPRC3B = 19.5384;            
            dSRM = 0.0749;
            dSRC12 = 15.7586; dSRC23 = 15.4435 + dSRC23Corr; dSRC3B = 19.3661;            
            dCP = 0.10;  dITM = dCP + 0.2; dBSCPX = 4.8046;  dBSCPY = 4.8497; dCPITM = 0.005;
            thetaPR2 = 0.79/180*pi*angleScale;  thetaPR3 = 0.615/180*pi*angleScale;
            thetaSR2 = 0.87/180*pi*angleScale;  thetaSR3 = 0.785/180*pi*angleScale;
            
            SRMRoC = -5.715;
            SRM2RoC = -6.406;
            SRM3RoC = 35.97;

            PRCAve = (dBSCPX+dBSCPY)/2 + dCPITM;
            PRCAsym = dBSCPX - dBSCPY;                       
            
            % test mass spec
            [ITMXdat, RoC_ITMX] = TMInfo( 'ITM', ITMXID, dirName );
            [ITMYdat, RoC_ITMY] = TMInfo( 'ITM', ITMYID, dirName );
            [SPTITMXdat, RoC_SPTITMX] = TMInfo( 'SPTITM', ITMXID, dirName );
            [SPTITMYdat, RoC_SPTITMY] = TMInfo( 'SPTITM', ITMYID, dirName );
            [ETMXdat, RoC_ETMX, T_ETMX, lossEHX] = TMInfo( 'ETM', ETMXID, dirName );
            [ETMYdat, RoC_ETMY, T_ETMY, lossEHY] = TMInfo( 'ETM', ETMYID, dirName );
            if pwr1DY > 0 || FO0 > 0
                ETMYdat = '';
                extraLossY = 3e-6;
                fprintf(1, 'ETMY data is empty because fit1dData is used\n');
            else
                extraLossY = 0;
            end
            [CPXdat, RoC_CPX] = TMInfo('CP', CPXID, dirName );
            [CPYdat, RoC_CPY] = TMInfo('CP', CPYID, dirName );
            
            if posIXX ~= 0 || posIXY ~= 0 || posEXX ~= 0 || posEXY ~= 0
                [thetaXI, phiXI, thetaXE, phiXE] = cavityManager.beam2tilt( posIXX, posIXY, posEXX, posEXY,1/1934, 1/RoC_ETMX, Larm)
            end
            if posIYX ~= 0 || posIYY ~= 0 || posEYX ~= 0 || posEYY ~= 0
                [thetaYI, phiYI, thetaYE, phiYE] = cavityManager.beam2tilt( posIYX, posIYY, posEYX, posEYY,1/1934, 1/RoC_ETMY, Larm)
            end
            
            if TMMap ~= 0
                armLoss = 1;
                armLossPPM = 0;
            else
                armLoss = 0;
                armLossPPM = 35e-6;
            end
            
            % add optics
            obj.addLaser( 'src' );
            obj.addMirror( 'PRM', 'invRoC', -1/11.009, 'Thick', dPRM, ...
                'T', 3.1e-2, 'Lhr', (15+0.5+PRCLoss)*1e-6*rmloss, 'Lar', (11.45+26)*1e-6*rmloss, 'Aperture', Apr1);
            obj.addMirror( 'PRM2', 'invRoC', -1/4.545, ...
                'T', 243e-6, 'Lhr', 8.6e-6*rmloss, 'inc', thetaPR2, 'Aperture', Apr2, 'removeCurvature', remC );
            obj.addMirror( 'PRM3', 'invRoC', 1/36.027, 'Aperture', Apr3, ...
                'T', 5.3e-6, 'Lhr', 17e-6*rmloss, 'incAngle', thetaPR3, 'removeCurvature', remC );
            
            obj.addMirror( 'SRM', 'invRoC', 1/SRMRoC, 'Thick', dSRM, ...
                'T', TSRM, 'Lhr', 7.6e-6*rmloss, 'Aperture', ASRM );
            obj.addMirror( 'SRM2', 'invRoC', 1/SRM2RoC, ...
                'T', 7.5e-6, 'Lhr', 12e-6*rmloss, 'inc', thetaSR2, 'Aperture', ASRM2 );
            obj.addMirror( 'SRM3', 'invRoC', 1/SRM3RoC, ...
                'T', 2.6e-6, 'Lhr', 21e-6*rmloss, 'incAngle', thetaSR3, 'Aperture', ASRM3 );

            lossIHX = 10.4e-6;
            lossIHY = 14.3e-6;
%             lossIHX = (lossIHX + lossIHY)/2;
%             lossIHY = lossIHX;
            lossIAX = (164+42)*1e-6;
            lossIAY = (250+80)*1e-6;
%             lossIAX = (lossIAX + lossIAY)/2;
%             lossIAY = lossIAX;
%            lossEHX = 10.9e-6;
%            lossEHY =  9.3e-6;
%             lossEHX = (lossEHX + lossEHY)/2;
%             lossEHY = lossEHX;

            obj.addMirror( 'BS', 'Aperture', Abs, 'Thickness', dBS, ...
                'T', 0.5, 'Lhr', 8.6e-6*Asym, 'Lar', 31.7e-6*Asym, 'incAngle', pi/4 );
            obj.addMirror( 'ITMX', 'invRoC', 1/1934, 'Aperture', Atm, 'Thick', dITM, ...
                'T', T_ITM, 'Lhr', Asym*(lossIHX*armLoss+armLossPPM/2)+extraArmLoss/2, 'Lar', Asym*lossIAX*ITMloss, ...
                'invFocal', invFocalX, 'optTheta', thetaXI, 'optPhi', phiXI, 'removeCurvature', remCurv );
            obj.addMirror( 'ITMY', 'invRoC', 1/1934, 'Aperture', Atm, 'Thick', dITM, ...
                'T', T_ITM, 'Lhr', Asym*(lossIHY*armLoss+armLossPPM/2)+extraArmLoss/2, 'Lar', Asym*lossIAY*ITMloss, ...
                'invFocal', invFocalY, 'optTheta', thetaYI, 'optPhi', phiYI, 'removeCurvature', remCurv );
            
            obj.tiltOn( 'ITMX', -posEXX/Larm, posEXY/Larm );
            obj.tiltOn( 'ITMY', -posEYX/Larm, posEYY/Larm );
            
            obj.addDetector( 'dark' );
            
            % build BS maps
            
            switch BSID
                case 2
                    obj.setHRfiles( 'BS', strcat(dirName,'BS02_S1_CIT.dat'), 'map+rsq/2*invRoCBS', 0.16, 0, 1, 'invRoCBS', invRoCBS );
                    obj.setARfiles( 'BS', strcat(dirName,'BS02_R2Fig.dat'), '', 0.16, 0, -1 );
                    obj.setTRfiles( 'BS', strcat(dirName,'BS02_SPTWE.dat'), '', 0.16, 0, -1 );
                case 5  % for L1
                    obj.setHRfiles( 'BS', strcat(dirName,'BS05_S1_CIT.dat'), 'map+rsq/2*invRoCBS', 0.16, 0, 1, 'invRoCBS', invRoCBS );
                    obj.setARfiles( 'BS', strcat(dirName,'xBS_05_R2_Figure.dat'), '', 0.16, 0, -1 );
                    obj.setTRfiles( 'BS', strcat(dirName,'xBS_05_SPTWE_Figure.dat'), '', 0.16, 0, -1 );
                case 6  % for H1
                    obj.setHRfiles( 'BS', strcat(dirName,'BS06_S1_CIT.dat'), 'map+rsq/2*invRoCBS', 0.16, 0, 1, 'invRoCBS', invRoCBS );
                    obj.setARfiles( 'BS', strcat(dirName,'xBS_06_R2_Figure.dat'), '', 0.16, 0, -1 );
                    obj.setTRfiles( 'BS', strcat(dirName,'xBS_06_SPTWE_Figure.dat'), '', 0.16, 0, -1 );
                case 100
                    obj.setRPYfiles( 'BS', strcat(dirName,'BS02_RPY.dat'), '', 0.22, 0 );
                    obj.setRSXfiles( 'BS', strcat(dirName,'BS02_RSX.dat'), '', 0.22, 0 );
                    obj.setTPXfiles( 'BS', strcat(dirName,'BS02_TPX.dat'), 'map(:,1:end-5)=map(:,6:end);func=map;', 0.22, 0 );
                    obj.setTSYfiles( 'BS', strcat(dirName,'BS02_TSY.dat'), 'map(:,1:end-5)=map(:,6:end);func=map;', 0.22, 0 );
                case 101
                    global BSmodel;
                    BSmodel = mphload('BS.mph');
                    obj.setRPYfiles( 'BS', '', '-sqrt(2)*calcBSatZ( ''w'', x(1,end:-1:1), y(:,1), 0, 0 );', 0.22, 0 );
                    obj.setRSXfiles( 'BS', '', '-(calcBST( x(1,:), y(:,1), 0.06, 10, 1 )+calcBST( x(1,:), y(:,1), 0.06, 10, -1 ));', 0.22, 0 );
                    obj.setTPXfiles( 'BS', '', '-calcBST( x(1,:), y(:,1), 0.06, 10, 1 );', 0.22, 0 );
                    obj.setTSYfiles( 'BS', '', '-calcBST( x(1,end:-1:1), y(:,1), 0.06, 10, -1 );', 0.22, 0 );
            end
            
            % implement BS baffle
            if useBSbaf
                var = sprintf('BFdx = %g; BFdy = %g; BSBFX = %g; BSBFY = %g;', BFdx, BFdy, BSBFX, BSBFY);
                obj.setModMap( 'BS', @obj.BSbaffle, var );
            end
            
            % ETMX
            obj.addMirror( 'ETMX', 'invRoC', 1/ETMRoC0, 'Aperture', Atm, 'Thick', 0.2, ... 
                'T', T_ETMX, 'Lhr', Asym*(armLoss*lossEHX+armLossPPM/2)+extraArmLoss/2, 'resolution', res, ...
                'optTheta', thetaXE, 'optPhi', phiXE, 'removeCurvature', remCurv, 'removeTilt', remTilt);
            % ETMY
            obj.addMirror( 'ETMY', 'invRoC', 1/ETMRoC0, 'Aperture', Atm, 'Thick', 0.2, ...
                'T', T_ETMY, 'Lhr', Asym*(armLoss*lossEHY+armLossPPM/2)+extraArmLoss/2+extraLossY, 'resolution', res, ...
                'optTheta', thetaYE, 'optPhi', phiYE, 'removeCurvature', remCurv, 'removeTilt', remTilt);
            
            
            CPXdat = '/Users/hiro/Dropbox/RichardDay/FOGPrime13/IFOanal/Aidan2018-11-13/WF_data_example_H1ITMX_1181476789.hdf5';
            CPYdat = '/Users/hiro/Dropbox/RichardDay/FOGPrime13/IFOanal/Aidan2018-11-13/WF_data_example_H1ITMY_1181476789.hdf5';
            wI = 0.053;
            wE = 0.062;                            
            
            %% test mass maps
            if isempty( TMMap ) || TMMap ~= 0
                if isempty( TMMap )
                    TMMap = 0;
                end
                % ITMX
                obj.setHRfiles( 'ITMX', ...
                    ITMXdat, ...     % phase map data file
                    'map*TMMap.*mapWeight(x,y,mr,mw,mNI) + calcThermal(0, r, 0, PcoatIX, wI, 0.2, 0.175 )', ...   % formula to modify the map : change the RoC for simple TCS
                    0.16, RoC_ITMX, 0, ...                % BS RoC, 1937.9m, was measured in the central 16cm
                    'PcoatIX', PcoatIX, 'wI', wI, 'TMMap', TMMap, 'mw', mw, 'mr', mr, 'mNI',mNI);                       % default value of the parameter in the formula
                obj.setTRfiles( 'ITMX', SPTITMXdat, 'map*TMMap - removePower(calcThermal(1, r, 0, PcoatIX, wI, 0.2, 0.175 ), x, y, 0.16, remAst)', ...
                    0.16, RoC_SPTITMX, 0, 'PcoatIX', PcoatIX*absCP, 'wI', wI, 'TMMap', TMMap, 'remAst', remAst);
%                obj.setTRfiles( 'ITMX', CPXdat, '', 0.16, RoC_CPX);
                
                % ITMY
                obj.setHRfiles( 'ITMY', ...
                    ITMYdat, ...     % phase map data file
                    'map*TMMap.*mapWeight(x,y,mr,mw,mNI) + calcThermal(0, r, 0, PcoatIY, wI, 0.2, 0.175 )', ...
                    0.16, RoC_ITMY, 0, ...
                    'PcoatIY', PcoatIY, 'wI', wI, 'TMMap', TMMap, 'mw', mw, 'mr', mr, 'mNI',mNI);   % ITMY RoC 1940.7m, was measured in the central 16cm
                obj.setTRfiles( 'ITMY', SPTITMYdat, ...
                    'map*TMMap - removePower(calcThermal(1, r, 0, PcoatIY, wI, 0.2, 0.175 ), x, y, 0.16, remAst)', 0.16, RoC_SPTITMY, 0, ... % set RoC aperture to 0.16 to use CP thermal lens
                    'PcoatIY', PcoatIY*absCP, 'wI', wI, 'TMMap', TMMap, 'remAst', remAst);
%                obj.setTRfiles( 'ITMY', CPYdat, '', 0, RoC_CPY);
                
                if ITMAbs == 'X'
                    obj.setHRfiles( 'ITMX', CPYdat, '-map/30*mapHR', fitR, 1e20, 0, 'mapHR', mapHR );
                    obj.setTRfiles( 'ITMX', CPXdat, '-map/2*mapOP', fitR, 1e20, 0, 'mapOP', mapOP );
                end
                
                if ITMAbs == 'Y'
                    obj.setHRfiles( 'ITMY', CPYdat, '-map/30*mapHR', fitR, 1e20, 0, 'mapHR', mapHR );
                    obj.setTRfiles( 'ITMY', CPYdat, '-map/2*mapOP', fitR, 1e20, 0, 'mapOP', mapOP );
                end
                                
                obj.setHRfiles( 'ETMX', ETMXdat, 'map*TMMap.*mapWeight(x,y,mr,mw,mNE) + calcThermal(0, r, 0, PcoatEX, wE, 0.2, 0.175 ) + pointAbs(x,y,xPoX, yPoX, PpointEX, absSizeX )+SDvals(uniptX,tind,x,y)', ...
                    0.16, 1/(1/RoC_ETMX+DinvRoCETM), 0, 'PcoatEX', PcoatEX, 'wE', wE, 'TMMap', TMMap, 'xPoX', xPoX, 'yPoX', yPoX, 'PpointEX', PpointEX, 'absSizeX', absSizeX, 'uniptX', uniptX, 'tind', tind, 'mw', mw, 'mr', mr, 'mNE', mNE);
                obj.setHRfiles( 'ETMY', ETMYdat, 'map*TMMap.*mapWeight(x,y,mr,mw,mNE) + rsq/2*DInvREY + calcThermal(0, r, 0, PcoatEY, wE, 0.2, 0.175 ) + pointAbs(x,y,xPoY, yPoY, PpointEY, absSizeY )+SDvals(uniptY,tind,x,y)+pwr1DY*fit1dData(oneDdat, x, y, -0.17)-falloff( x, y, FO0, 0.17, 140e-9, NFO)', ...
                    0.16, 1/(1/RoC_ETMY+DinvRoCETM), 0, 'PcoatEY', PcoatEY, 'wE', wE, 'TMMap', TMMap, 'xPoY', xPoY, 'yPoY', yPoY, 'PpointEY', PpointEY, 'absSizeY', absSizeY, 'uniptY', uniptY, 'tind', tind, 'mw', mw, 'mr', mr, 'mNE', mNE, ...
                    'DInvREY', DInvREY, 'pwr1DY', pwr1DY, 'oneDdat', oneDdat, 'FO0', FO0, 'NFO', NFO );
            end
            
            %% propagators
            % arms
            DCphase = DCoffset/1.064e-6*2*pi;
            obj.addProp( '', 'ITMX-HR', 'ETMX-HR', Larm, 0, -DCphase/2 );
            obj.addProp( '', 'ITMY-HR', 'ETMY-HR', Larm, 0, DCphase/2 );

            obj.addProp( '', 'src', 'PRM-AR' );
            obj.addProp( '', 'PRM-HR', 'PRM2-HRx', dPRC12, 0, 0 );
            obj.addProp( 'Prop23', 'PRM2-HRy', 'PRM3-HRx', dPRC23 );
            obj.addProp( '', 'PRM3-HRy', 'BS-HRx', dPRC3B );
            
            obj.addProp( '', 'BS-ARx', 'ITMX-AR', PRCAve + PRCAsym/2);
            obj.addProp( '', 'BS-HRy', 'ITMY-AR', PRCAve - PRCAsym/2);
            
            % SRC props : uncomment this to add SRC
            obj.addProp( '', 'SRM', 'SRM2-HRx', dSRC12, 0, phiSRC );
            obj.addProp( 'PropS23', 'SRM2-HRy',  'SRM3-HRx', dSRC23 );
            obj.addProp( '', 'SRM3-HRy',  'BS-ARy', dSRC3B );
            obj.addProp( '', 'SRM-AR',  'dark' );
            % make sure the following one to be commented out when SRC is added
%            obj.props.addProp( '', 'BS-ARy', 'dark');
        end
        
        function setupBaseMode( obj, varargin )
            
            useArmMode = 1;
            dz = 0;
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            if useArmMode == 0 || obj.optics.findNamedOptic('ETMX') <= 0
                qInv = 1.318657072498640e-01 - 6.818783109537356e-02i;
                qInv = 1/(1/qInv + dz);
                rootID = 2;
                %                 cav = obj.cavities.closedCavs{1};
                %                 ind = find( cav(2:end) == cav(1), 1 );
                %                 cav = cav(1:ind+1);
                %                 [qInv, rootID] = obj.cavities.findCavityMode( cav );
                %              rootID = 1;
                %              qInv = -0.132522 - i*0.0332622;
                %               -1.323621960182562e-01 - 6.849822106634568e-02i
            else
                [invQXP, invQXS] = obj.fillQvalFromArm2BS( 'X' );
                [invQYP, invQYS] = obj.fillQvalFromArm2BS( 'Y' );
            
                % SRC
                invQfromBS = ( invQXS + invQYS ) / 2;
                obj.fields.setQInv('BS-ARy-o', invQfromBS);
                obj.fields.setQInv('BS-ARy-i', invQfromBS, true);
                
                L23Adj = MMTK.adjustL4RoC( obj.optics.optList(obj.optics.findNamedOptic('SRM')).invRoC, ...                    
                    obj.cavities.findCavityABCD( {'SRM2-HRy-i', 'SRM2-HRx-o', 'SRM-HR-i'} ), ...
                    obj.cavities.findCavityABCD( {'BS-ARy-o', 'SRM3-HRx-o'} ), invQfromBS );

                L0 = obj.props.getPropLeng( 'PropS23' );
                if abs(L23Adj(1)-L0) < abs(L23Adj(2)-L0)
                    L0 = L23Adj(1);
                else
                    L0 = L23Adj(2);
                end
                obj.props.setL02( 'PropS23',  L0 );
                
                % PRC
                % invQ from BS
                invQfromBS = ( invQXP + invQYP ) / 2;
                obj.fields.setQInv('BS-HRx-o', invQfromBS);
                obj.fields.setQInv('BS-HRx-i', invQfromBS, true);
                
                L23Adj = MMTK.adjustL4RoC( obj.optics.optList(obj.optics.findNamedOptic('PRM')).invRoC, ...                    
                    obj.cavities.findCavityABCD( {'PRM2-HRy-i', 'PRM2-HRx-o', 'PRM-HR-i'} ), ...
                    obj.cavities.findCavityABCD( {'BS-HRx-o', 'PRM3-HRx-o'} ), invQfromBS );

                L0 = obj.props.getPropLeng( 'Prop23' );
                if abs(L23Adj(1)-L0) < abs(L23Adj(2)-L0)
                    L0 = L23Adj(1);
                else
                    L0 = L23Adj(2);
                end
                obj.props.setL02( 'Prop23',  L0 );
                
                qInv = invQfromBS;
                obj.cavities.fillCavityInvQ( {'BS-HRx-o', 'PRM3-HRx-o', 'PRM2-HRx-o', 'PRM-HR-i'}, qInv, true );
                rootID = obj.optics.findPortID( 'BS-HRx-o' );
            end
            
            obj.cavities.fillQvals( rootID, qInv );                        
        end

        function AccSetup( obj, AccG, doLock, delLGain, varargin )
            
            SRCLock = true;
            SRCphase = 0;
            eval( UtilTK.setVars( who, true, varargin{:} ) );
            
            PRMHR_ID = obj.fields.findNamedField( 'PRM-HR-o' );
            SRMHR_ID = obj.fields.findNamedField( 'SRM-HR-o' );
            ITMXHR_ID = obj.fields.findNamedField( 'ITMX-HR-o' );
            ITMYHR_ID = obj.fields.findNamedField( 'ITMY-HR-o' );
            PRM2BS = {'PRM-HR-o', 'PRM2-HRy-o', 'PRM3-HRy-o'};
            SRM2BS = {'SRM-HR-o', 'SRM2-HRy-o', 'SRM3-HRy-o'};
            BS2Y = {'BS-HRy-o', 'ITMY-AR-o'}; BS2X = {'BS-ARx-o', 'ITMX-AR-o'};
            BS2PRM = {'BS-HRx-o', 'PRM3-HRx-o', 'PRM2-HRx-o', 'PRM-HR-o'};
            BS2SRM = {'BS-ARy-o', 'SRM3-HRx-o', 'SRM2-HRx-o', 'SRM-HR-o'};
            ITMX2ETMX = {'ITMX-HR-o', 'ETMX-HR-o'};
            ITMY2ETMY = {'ITMY-HR-o', 'ETMY-HR-o'};

            % index = AccGuessObj.defineGuess( obj, guessID, convID, lockProp, lockPhase, pathList, delLG )
            % lockProp < 0 means do not change for locking
            % lockPhase NAN means to use the locked phase specified in addProp
            AccG.defineGuess( PRMHR_ID, PRMHR_ID, obj.props.getPropID(PRMHR_ID), 0, ...
                { [PRM2BS, BS2Y, BS2PRM], [PRM2BS, BS2X, 'BS-HRx-o'], ...
                  [PRM2BS, BS2Y, BS2SRM], [PRM2BS, BS2X, 'BS-ARy-o'], ...
                  [PRM2BS, 'BS-HRy-o', 'ITMY-HR-o'], ...
                  [PRM2BS, 'BS-ARx-o', 'ITMX-HR-o'] }, ...
                  delLGain );
              
            SRCProp = obj.props.getPropID(SRMHR_ID);
            if ~SRCLock
                SRCProp = -SRCProp;
            end
            AccG.defineGuess( SRMHR_ID, SRMHR_ID, SRCProp, SRCphase, ...
                { [SRM2BS, BS2Y, BS2SRM], [SRM2BS, BS2X, 'BS-ARy-o'], ...
                  [SRM2BS, BS2Y, BS2PRM], [SRM2BS, BS2X, 'BS-HRx-o'], ...
                  [SRM2BS, 'BS-HRy-o', 'ITMY-HR-o'], ...
                  [SRM2BS, 'BS-ARx-o', 'ITMX-HR-o']}, ...
                  delLGain );
              
            AccG.defineGuess( ITMXHR_ID, ITMXHR_ID, obj.props.getPropID(ITMXHR_ID), 0, ...
                { [ITMX2ETMX, 'ITMX-HR-o'], ...
                  [ITMX2ETMX, 'ITMX-AR-o', BS2PRM], ...
                  [ITMX2ETMX, 'ITMX-AR-o', BS2SRM]}, ...
                  delLGain );

            AccG.defineGuess( ITMYHR_ID, ITMYHR_ID, obj.props.getPropID(ITMYHR_ID), 0, ...
                { [ITMY2ETMY, 'ITMY-HR-o'], ...
                  [ITMY2ETMY, 'ITMY-AR-o', BS2PRM], ...
                  [ITMY2ETMY, 'ITMY-AR-o', BS2SRM] }, ...
                  delLGain );
        end
        
    end
    
    methods( Static )
        % implement a BS baffle
        function [reflOut, transOut] = BSbaffle( reflIn, transIn, x, y, var )
            
            eval( var );
            baffleMask = (x-BFdx).^2/BSBFX^2+(y-BFdy).^2/BSBFY^2 < 1;
            
            reflOut{1} = baffleMask .* reflIn{1};
            reflOut{2} = reflIn{2} .* fliplr(baffleMask);
            reflOut{3} = reflIn{3} .* fliplr(baffleMask);
            reflOut{4} = baffleMask .* reflIn{4};
            transOut{1} = baffleMask .* transIn{1};
            transOut{2} = transIn{2} .* fliplr(baffleMask);
            transOut{3} = transIn{3} .* fliplr(baffleMask);
            transOut{4} = baffleMask .* transIn{4};
        end
    end
end
